;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME:   INSAMP_1.asm
;;  Version: 2.2, Updated on 2006/05/18 at 10:31:38
;;  Generated by PSoC Designer ver 4.4  b1884 : 14 Jan, 2007
;;
;;  DESCRIPTION:
;;    Assembler source code for a three-OpAmp Instrumentation Amplifier
;;    topology in the 22/24/27/28xxx family of PSoC devices
;;
;;  NOTE: User Module APIs conform to the fastcall16 convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API functions
;;        returns. For Large Memory Model devices it is also the caller's 
;;        responsibility to perserve any value in the CUR_PP, IDX_PP, MVR_PP and 
;;        MVW_PP registers. Even though some of these registers may not be modified
;;        now, there is no guarantee that will remain the case in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "memory.inc"
include "INSAMP_1.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

export  INSAMP_1_Start
export _INSAMP_1_Start
export  INSAMP_1_SetPower
export _INSAMP_1_SetPower

export  INSAMP_1_Set2StageGain
export _INSAMP_1_Set2StageGain

export  INSAMP_1_Stop
export _INSAMP_1_Stop


AREA bss (RAM,REL)

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------

POWERMASK:                     equ 0x03    ; CT and SC block power mask
CT_RTAP:                       equ 0xF0    ; CT resistor tap mask
CT_EXGAIN:                     equ 0x01    ; CT gain expansion bit mask
SC_CAP:                        equ 0x1F    ; SC A, B & C cap bit mask
SC_FDBK:                       equ 0x80    ; SC F cap bit mask


;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------


AREA UserModules (ROM, REL)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: INSAMP_1_Start
;  FUNCTION NAME: INSAMP_1_Setpower
;
;  DESCRIPTION:
;    Sets power to the analog blocks used in this module. All three blocks
;    are set to the same power setting (which should be the same as the
;    analog reference power global parameter for best operation!). Constants
;    for proper values are defined in asm and C interface files.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    fastcall16 BYTE bPowerSetting in the A register
;  RETURNS:      Nothing
;  SIDE EFFECTS: 
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;
 INSAMP_1_Start:
_INSAMP_1_Start:
 INSAMP_1_SetPower:
_INSAMP_1_SetPower:
    RAM_PROLOGUE RAM_USE_CLASS_2
    and  A, POWERMASK                            ; trim arg to legal value
    mov  X, SP                                   ; initialize stack frame
    push A                                       ;
    mov  A, reg[INSAMP_1_INV_CR2]                ; read power value in inverting input block
    and  A, ~POWERMASK                           ; clear power bits
    or   A, [X]                                  ; combine power value with balance of reg
    mov  reg[INSAMP_1_INV_CR2], A                ;
    mov  A, reg[INSAMP_1_NON_INV_CR2]            ; repeat for non-inverting input block
    and  A, ~POWERMASK                           ;
    or   A, [X]                                  ;
    mov  reg[INSAMP_1_NON_INV_CR2], A            ;
    mov  A, reg[INSAMP_1_CONVERT_CR3]            ; repeat for SC converstion block
    and  A, ~POWERMASK                           ;
    or   A, [X]                                  ;
    mov  reg[INSAMP_1_CONVERT_CR3], A            ;
    pop  A                                       ; release stack frame
    RAM_EPILOGUE RAM_USE_CLASS_2
    ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: INSAMP_1_Set2StageGain
;
;  DESCRIPTION: Applies gain set values to both the input and output stages
;    of the amplifier.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    fastcall16 BYTE bInputGain  in the A register
;                         BYTE bOutputGain in the X register
;  RETURNS:      Nothing
;  SIDE EFFECTS: 
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;
;  THEORY OF OPERATION: The total gain of the amplifier is the product of the
;    input and output gains. The input gain value configures the resistor
;    string in the continuous-time blocks that form the differential section.
;    The output gain configures the A, B and F capacitors of the differential
;    to single- ended conversion stage which uses a switched capacitor PSoC
;    block. Symbolic constants for the input and output gains are defined in
;    the assembly and C interface files. Each constant encodes values for
;    multiple bit fields as follows:
;     INGAIN[7:4]  holds ACBxxCR0[7:4] (RTapMux of both CT blocks)
;     INGAIN[0]    holds ACBxxCR3[1]   (EXGAIN  of both CT blocks)
;     OUTGAIN[7]   holds ASxyyCR0[7]   (FCap of the SC block)
;     OUTGAIN[4:0] holds ASyxxCR0[4:0] (ACap of the SC block)
;                  *AND* ASyxxCR1[4:0] (BCap of the SC block)
;
;  STACK FRAME:         Offset
     INGAIN:            equ  0
     OUTGAIN:           equ -1
     GAIN_FRAME_SIZE:   equ  2

 INSAMP_1_Set2StageGain:
_INSAMP_1_Set2StageGain:
    RAM_PROLOGUE RAM_USE_CLASS_2
    push  X                                            ; Set up stack frame
    mov   X, SP                                        ;   and pointer
    push  A                                            ;
    and   A, ~CT_RTAP                                  ; Is Gain of 48 or 24 required?
    jz    .NormalInputGainRange                        ;   No, go clear EXGAIN bit
    or    reg[INSAMP_1_INV_CR3],     CT_EXGAIN         ;   Yes, set EXGAIN in the
    or    reg[INSAMP_1_NON_INV_CR3], CT_EXGAIN         ;       difference blocks
    jmp   .SetResistorTap

.NormalInputGainRange:
    and   reg[INSAMP_1_INV_CR3],     ~CT_EXGAIN        ; Clear EXGAIN bit in the
    and   reg[INSAMP_1_NON_INV_CR3], ~CT_EXGAIN        ;       difference blocks

.SetResistorTap:
    and   [X+INGAIN], CT_RTAP                          ; Trim INGAIN to resistor value
    mov   A, reg[INSAMP_1_NON_INV_CR0]                 ; Set resistor tap bit field
    and   A, ~CT_RTAP                                  ;   for the non-inverting block
    or    A, [X+INGAIN]                                ;
    mov   reg[INSAMP_1_NON_INV_CR0], A                 ;
    mov   A, reg[INSAMP_1_INV_CR0]                     ; Set resistor tap bit field
    and   A, ~CT_RTAP                                  ;   for the inverting block
    or    A, [X+INGAIN]                                ;
    mov   reg[INSAMP_1_INV_CR0], A                     ;

    mov   A, reg[INSAMP_1_CONVERT_CR0]                 ; Grab ACap/FCap values
    tst   [X+OUTGAIN], SC_FDBK                         ; Is Output Gain >= 1?
    jnz   .SetOutputForLoss                            ;   No,  go set FCap to 32
    and   A, ~SC_FDBK                                  ;   Yes,    set FCap to 16
    jmp   .SetSCInputCaps

.SetOutputForLoss:
    or    A, SC_FDBK                                   ; FCap <- 32

.SetSCInputCaps:
    and   [X+OUTGAIN], SC_CAP                          ; Trim OUTGAIN to Cap Value
    and   A, ~SC_CAP                                   ; Set conversion stage ACap
    or    A, [X+OUTGAIN]                               ;   (couples the NON-INV
    mov   reg[INSAMP_1_CONVERT_CR0], A                 ;      block input)
    mov   A, reg[INSAMP_1_CONVERT_CR1]                 ; Set the BCap (coupling the
    and   A, ~SC_CAP                                   ;   INV block) to the same
    or    A, [X+OUTGAIN]                               ;   value as the ACap
    mov   reg[INSAMP_1_CONVERT_CR1], A                 ;   ...
    add   SP, -GAIN_FRAME_SIZE                         ; release stack frame
    RAM_EPILOGUE RAM_USE_CLASS_2
    ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: INSAMP_1_Stop
;
;  DESCRIPTION: Turns off power to the Analog blocks
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: 
;    The A and X registers may be modified by this or future implementations
;    of this function.  The same is true for all RAM page pointer registers in
;    the Large Memory Model.  When necessary, it is the calling function's
;    responsibility to perserve their values across calls to fastcall16 
;    functions.
;
 INSAMP_1_Stop:
_INSAMP_1_Stop:
    RAM_PROLOGUE RAM_USE_CLASS_1
    and  reg[INSAMP_1_CONVERT_CR3], ~POWERMASK
    and  reg[INSAMP_1_NON_INV_CR2], ~POWERMASK
    and  reg[INSAMP_1_INV_CR2],     ~POWERMASK
    RAM_EPILOGUE RAM_USE_CLASS_1
    ret

.ENDSECTION

; End of File INSAMP_1.asm
