//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------
#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>
#include "motorcontroller.h"
#include "rx8_1.h"
#include <psocgpioint.h>

// Global Variables

int 	TimeSlot = 0;			// Scheduler variable
int 	SliceCounter = 0;		// Scheduler variable

// Main Program

void main()
{

// Variables

	PIDstruct	SouthPID;
	PIDstruct	NorthPID;
	CmdStruct	Command;
	WheelStruct	NorthWheel;
	WheelStruct SouthWheel;

	BYTE		bRxStatus;
	BYTE		bTxStatus;
	BYTE		CmdRxByte = 0;			// Rx interrupt Variables
	BYTE		StsTxByte = 0;			// Tx interrupt Variables
	BYTE		LS_Mask = 0;
	BYTE		R_LS_Mask = 0;
	BYTE		L_LS_Mask = 0;
	BYTE		Mask = 0;
	BOOL		Stop;
	BOOL		SetTare;
	BOOL		PSoC_Select;
	BOOL		Weigh;

	unsigned long  	ADCsample = 0;                  // ADC sample
	unsigned long	ADCsum = 0;
	unsigned long	ADCavg;
	unsigned long	adccnt = 0;
	int				tare = 0;
	int				weight;
	char 	       	str[17];			            // One line of characters for LCD
	char		   	blank[] = "                ";	// Clear a line of LCD
	
	int 			pwr[4];
	int				cnt;

// Initialization

	initPSoC(); 						// Init PSoC functions
	StopMotor(BOTH);
	PIDinit(&SouthPID);					// Initialize the PID control loops
	PIDinit(&NorthPID);
	StsTxByte |= PSOCSTOP;

	NorthWheel.id = NORTH;				// Initialize Wheel Structs
	SouthWheel.id = SOUTH;
	Command.Status = NEW;				// Initialize Command Structs
	Command.Velocity = 0;
	Command.error = 0;
	Command.wReadCounter = 0;
	Command.END_SENSOR = FALSE;
	Command.PWMvalue = 0;

	Stop = FALSE;
	SetTare = TRUE;
	PSoC_Select = FALSE;
	Weigh = FALSE;

	pwr[0] = 1;
	pwr[1] = 2;
	pwr[2] = 4;
	pwr[3] = 8;
	
	while(TRUE)
	{
// Do this every 10 ms
// Check to see if we got a byte from XBC.  If so check to see
// if data or command byte.  Then go ahead and process the byte.

		if (TimeSlot & 2)
		{
			TimeSlot ^= 2;

			if (bRxStatus = RX8_1_bReadRxStatus() & RX8_RX_COMPLETE)
			{
				CmdRxByte = RX8_1_bReadRxData();

				if ((CmdRxByte | NS_PSOC) == NS_PSOC)
				{
					PSoC_Select = TRUE;
					TX8_1_Start(TX8_PARITY_NONE);
					ProcessRxByte(CmdRxByte, &Command);
					StsTxByte &= NS_PSOC;
					StsTxByte &= PSOCSTART;
					StsTxByte |= STSWORD;

					if (Command.Velocity == 0)
					{
						Weigh = TRUE;
						Stop = TRUE;
					}

					SouthWheel.position = PRESET_CNT;
					NorthWheel.position = PRESET_CNT;
				}

				bRxStatus = 0;
				RX8_1_CmdReset();
			}

			if ((PSoC_Select) && (CmdRxByte & ALIGN) && (Command.Velocity > 0) && !(CmdRxByte & MTYPE))
			{
				LS_Mask = (PRT2DR | IS_LN_MSK);

				switch (LS_Mask)
				{
					case LS_MSK6:
						Stop = TRUE;
					break;

					case LS_MSK7:
						Stop = TRUE;
					break;

					case LS_MSK8:
						Stop = TRUE;
					break;

					case LS_MSK9:
						Stop = TRUE;
					break;

					case IS_LN_MSK:
						Stop = TRUE;
					break;

					default:
						Stop = FALSE;
				}
			}

			else if ((PSoC_Select) && (CmdRxByte & IS_DT) && (Command.Velocity > 0) && !(CmdRxByte & MTYPE))
			{
				LS_Mask = (PRT2DR | IS_LN_MSK);

				switch (LS_Mask)
				{
					case LS_MSK1:
						Stop = TRUE;
					break;

					case LS_MSK2:
						Stop = TRUE;
					break;

					case LS_MSK3:
						Stop = TRUE;
					break;

					case LS_MSK4:
						Stop = TRUE;
					break;

					case LS_MSK5:
						Stop = TRUE;
					break;

					case LS_MSK6:
						Stop = TRUE;
					break;

					case LS_MSK7:
						Stop = TRUE;
					break;

					case LS_MSK8:
						Stop = TRUE;
					break;

					case LS_MSK9:
						Stop = TRUE;
					break;

					case LS_MSK10:
						Stop = TRUE;
					break;

					case LS_MSK11:
						Stop = TRUE;
					break;

					case LS_MSK12:
						Stop = TRUE;
					break;

					case LS_MSK13:
						Stop = TRUE;
					break;

					case LS_MSK14:
						Stop = TRUE;
					break;

					case LS_MSK15:
						Stop = TRUE;
					break;

					case LS_MSK16:
						Stop = TRUE;
					break;

					case LS_MSK17:
						Stop = TRUE;
					break;

					case LS_MSK18:
						Stop = TRUE;
					break;

					case IS_LN_MSK:
						Stop = TRUE;
					break;

					default:
						LS_Mask = (PRT2DR & IS_LN_MSK);
						if (!LS_Mask)
						{
							Stop = TRUE;
						}

						else
						{
							Stop = FALSE;
						}
				}
			}
			
			if ((PSoC_Select) && (Command.Velocity > 0) && (CmdRxByte & MTYPE))
			{
				LS_Mask = (PRT2DR | IS_LN_MSK);

				switch (LS_Mask)
				{
					case IS_LN_MSK:
						Stop = TRUE;
					break;

					default:
						Stop = FALSE;
				}
			}

			if ((PSoC_Select && Stop) || (PSoC_Select && Command.Velocity == 0))
			{
				StopMotor(BOTH);

	       		Command.Velocity = 0;
	       		Command.VP6 = 0;
	   			Command.VP5 = 0;
	   			Command.VP4 = 0;
	   			Command.VP3 = 0;
	   			Command.VP2 = 0;
	   			Command.VP1 = 0;
	   			Command.VS1 = 0;
	   			Command.VS2 = 0;
	   			Command.VS3 = 0;
	   			Command.VS4 = 0;
	   			Command.VS5 = 0;
	   			Command.VS6 = 0;

	       		if (Stop)
	       		{
	       			StsTxByte |= PSOCSTOP;
	       			Stop = FALSE;
	       		}

	       		if (Command.PrevOffline)
	       		{

	       			StsTxByte |= LINELEFT;
	       		}

	       		else
	       		{
	       			StsTxByte &= LINERIGHT;
	       		}

	       		if (Command.Center)
	       		{
	       			StsTxByte |= CENTER;
	       		}
	       		
	       		else
	       		{
	       			StsTxByte &= NOTCENTER;
	       		}

	       		if (PSoC_Select)
	       		{
	       			TX8_1_SendData(StsTxByte);
					TX8_2_SendData(0x94);
					TX8_2_PutString(blank);
					TX8_2_SendData(0x94);
					TX8_2_PutSHexByte(StsTxByte);
					Weigh = FALSE;
					PSoC_Select = FALSE;
	       		}

				if (bTxStatus = bTX8_1_ReadTxStatus() & TX8_1_TX_COMPLETE)
				{
					TX8_1_Stop();
					bTxStatus = 0;
	       		}
			}
			
			LS_Mask = (PRT2DR | IS_LN_MSK);
			LS_Mask = ~LS_Mask;
			cnt = 3;
			R_LS_Mask = LS_Mask >> 4;
			L_LS_Mask = 0;
			
			while(cnt >= 0)
			{
				Mask = 0;
				LS_Mask = LS_Mask >> 1;
				Mask = (LS_Mask & IS_LN_MSK);
				Mask = Mask * pwr[cnt];
				L_LS_Mask = (L_LS_Mask | Mask);
				cnt--;
			}				
			
			Command.PrevOffline = Command.Offline;
			
			if (R_LS_Mask < L_LS_Mask)
			{
				Command.Offline = LEFT;
				Command.Center = FALSE;
				
				if (L_LS_Mask == 8)
				{
					Command.END_SENSOR = TRUE;
				}
				
				else
				{
					Command.END_SENSOR = FALSE;
				}
			}
			
			else if (R_LS_Mask > L_LS_Mask)
			{
				Command.Offline = RIGHT;
				Command.Center = FALSE;
				
				if (R_LS_Mask == 8)
				{
					Command.END_SENSOR = TRUE;
				}
			
				else
				{
					Command.END_SENSOR = FALSE;
				}
			}
			
			else
			{
				if (!Command.END_SENSOR)
				{
					Command.Center = TRUE;
				}
			}
		}// end 10 ms timeslot

//
// Do this every 20 ms
//
		if (TimeSlot & 4)
		{
			TimeSlot ^= 4;

			if(ADCINC14_1_fIsDataAvailable() != 0)		// See if data ready
            {
                ADCsample = ADCINC14_1_iGetData();     // Get Data
                ADCsum += ADCsample;
                adccnt++;
                ADCINC14_1_ClearFlag();                // Clear data ready flag
            }
            
			ADCINC14_1_GetSamples(1); 			        // Tell ADC to convert 1 sample (takes 16ms to convert)
		}// end 20 ms timeslot

// Do this every 40 ms (the sample period is 40 ms)

		if (TimeSlot & 8)
		{
			TimeSlot ^= 8;

			if (PSoC_Select)
			{
				ControlWheel(&Command, &SouthWheel, &SouthPID);
				ControlWheel(&Command, &NorthWheel, &NorthPID);
			}

		}// end 40 ms timeslot

// Do this every 160 ms

		if (TimeSlot & 32)
		{
			TimeSlot ^= 32;
		}// end 160 ms timeslot

		if (TimeSlot & 128)
		{
			TimeSlot ^= 128;

        } //  end 1.28 seconds

// Update the LCD screen once each 1.28 seconds

		if (TimeSlot & 256)
		{
			TimeSlot ^= 256;

			ADCavg = ADCsum / adccnt;
			ADCsum = 0;
			adccnt = 0;

			if (SetTare)
			{
				tare = ADCavg;
				SetTare = FALSE;
			}

			weight = 0.9567*(tare - ADCavg);
			TX8_2_SendData(0x80);
			TX8_2_PutString(blank);
 		    itoa(str,weight,10);   			// Convert an integer to ascii
			TX8_2_SendData(0x80);
			TX8_2_PutString(str);
			TX8_2_CPutString(" g");

			if ((weight > 30) && (weight < 93))
			{
				StsTxByte &= NOWEIGHT;
				StsTxByte |= LTWEIGHT;
			}

			else if ((weight > 94) && (weight < 156))
			{
				StsTxByte &= NOWEIGHT;
				StsTxByte |= MDWEIGHT;
			}

			else if (weight > 157)
			{
				StsTxByte &= NOWEIGHT;
				StsTxByte |= HVWEIGHT;
			}

			else
			{
				StsTxByte &= NOWEIGHT;
			}
		}// end 1280 ms timeslot
	}// end while loop
}// end of main program



////////////////////////////////////////////////////////////////////////

// Timer interrupt routine (every 5 ms)
#pragma interrupt_handler Timer16_1_ISR
#pragma nomac

void Timer16_1_ISR()
{
	int temp;

	temp = SliceCounter++;
	TimeSlot |= ((temp ^ SliceCounter) + 1);
}