#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>
#include "motorcontroller.h"

///////////////////////////////////////////////////////////////////////////////

// PSoC initialization routine

void initPSoC(void)
{

// Initialize the UART receiver
// No parity

	RX8_1_Start(RX8_PARITY_NONE);
	TX8_2_Start(TX8_PARITY_NONE);

// Enable PWM8_1 for motor 0 (left-side motor)

	PWM8_1_DisableInt();
	PWM8_1_WritePulseWidth(0);
	PWM8_1_Start();

// Enable PWM8_2 for motor 1 (right-side motor)

	PWM8_2_DisableInt();
	PWM8_2_WritePulseWidth(0);
	PWM8_2_Start();
	
//	Stop the 16 bit encoder counters

    Counter16_1_Stop();
    Counter16_2_Stop();	
	
// Turn on the 14 bit ADC

   ADCINC14_1_Start(ADCINC14_1_HIGHPOWER);   // Turn on ADC

// Start up the instrumentation amplifier

   INSAMP_1_Start(INSAMP_1_HIGHPOWER);
   INSAMP_1_Set2StageGain(INSAMP_1_INGAIN_48, INSAMP_1_OUTGAIN_1_94);	

// Enable timer interrupts needed for scheduling
	Timer16_1_EnableInt();
	Timer16_1_Start();
	Timer8_1_Start();

// Enable all interrupts
	M8C_EnableGInt;
	
	return;
}

////////////////////////////////////////////////////////////////////////

void Motor(int MotorCode, int PWMvalue, BOOL MotorDirection)
{										// PWMvalue 0-255
   if (MotorCode == NORTH) 				// North motor
   {
	   if(MotorDirection == FWD)		// Forward
	   {
		   PRT1DR |= NMC_R_ON;
		   PRT1DR &= NMC_B_OFF;
	   }

	   else 							// Backward
	   {
		   PRT1DR &= NMC_R_OFF;
		   PRT1DR |= NMC_B_ON;
	   }

	   PWM8_2_WritePulseWidth(PWMvalue);
   }

   else 								// South motor
   {
	   if(MotorDirection == FWD)		// Forward
	   {
		   PRT1DR |= SMC_R_ON;
		   PRT1DR &= SMC_B_OFF;
	   }

	   else 							// Backward
	   {
		   PRT1DR &= SMC_R_OFF;
		   PRT1DR |= SMC_B_ON;
	   }

	   PWM8_1_WritePulseWidth(PWMvalue);
   }

   return;
}

//////////////////////////////////////////////////////////////////////

// Stop Function
void StopMotor(int MotorCode)
{
	if (MotorCode == SOUTH)				// Stop South Motor
	{
		PRT1DR &= SMC_OFF;
	}

	if (MotorCode == NORTH)				// Stop North Motor
	{
		PRT1DR &= NMC_OFF;
	}

	if (MotorCode == BOTH)				// Stop both motors
	{
		PRT1DR &= BOTH_OFF;
	}

	PWM8_1_WritePulseWidth(0);
	PWM8_2_WritePulseWidth(0);

	return;
}


