#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdlib.h>
#include "motorcontroller.h"

void ProcessRxByte(BYTE RxByte, CmdStruct * Cmd)
{
	BYTE	VelocityCode;

	Cmd->MotionType = ((RxByte & 0x20) >> 5);
	VelocityCode = ((RxByte & 0x18) >> 3);
	if (RxByte & LF_ON)
		Cmd->LineFollow = 1;
		else
		Cmd->LineFollow = 0;


	switch (VelocityCode)
	{
	   	case 0x0:
	   		Cmd->Velocity = 0;
	   		Cmd->VP6 = 0;
	   		Cmd->VP5 = 0;
	   		Cmd->VP4 = 0;
	   		Cmd->VP3 = 0;
	   		Cmd->VP2 = 0;
	   		Cmd->VP1 = 0;
	   		Cmd->VS1 = 0;
	   		Cmd->VS2 = 0;
	   		Cmd->VS3 = 0;
	   		Cmd->VS4 = 0;
	   		Cmd->VS5 = 0;
	   		Cmd->VS6 = 0;
	    break;

	   	case 0x1:
	   		Cmd->Velocity = SLOW;
	   		Cmd->VP6 = SLOWP6;
	   		Cmd->VP5 = SLOWP5;
	   		Cmd->VP4 = SLOWP4;
	   		Cmd->VP3 = SLOWP3;
	   		Cmd->VP2 = SLOWP2;
	   		Cmd->VP1 = SLOWP1;
	   		Cmd->VS1 = SLOWS1;
	   		Cmd->VS2 = SLOWS2;
	   		Cmd->VS3 = SLOWS3;
	   		Cmd->VS4 = SLOWS4;
	   		Cmd->VS5 = SLOWS5;
	   		Cmd->VS6 = SLOWS6;
	    break;

	   	case 0x2:
	   		Cmd->Velocity = MEDIUM;
	   		Cmd->VP6 = MEDIUMP6;
	   		Cmd->VP5 = MEDIUMP5;
	   		Cmd->VP4 = MEDIUMP4;
	   		Cmd->VP3 = MEDIUMP3;
	   		Cmd->VP2 = MEDIUMP2;
	   		Cmd->VP1 = MEDIUMP1;
	   		Cmd->VS1 = MEDIUMS1;
	   		Cmd->VS2 = MEDIUMS2;
	   		Cmd->VS3 = MEDIUMS3;
	   		Cmd->VS4 = MEDIUMS4;
	   		Cmd->VS5 = MEDIUMS5;
	   		Cmd->VS6 = MEDIUMS6;
	    break;

	   	case 0x3:
	   		Cmd->Velocity = FAST;
	   		Cmd->VP6 = FASTP6;
	   		Cmd->VP5 = FASTP5;
	   		Cmd->VP4 = FASTP4;
	   		Cmd->VP3 = FASTP3;
	   		Cmd->VP2 = FASTP2;
	   		Cmd->VP1 = FASTP1;
	   		Cmd->VS1 = FASTS1;
	   		Cmd->VS2 = FASTS2;
	   		Cmd->VS3 = FASTS3;
	   		Cmd->VS4 = FASTS4;
	   		Cmd->VS5 = FASTS5;
	   		Cmd->VS6 = FASTS6;
	    break;

	   	default:
	   		Cmd->Velocity = 0;
	   		Cmd->VP6 = 0;
	   		Cmd->VP5 = 0;
	   		Cmd->VP4 = 0;
	   		Cmd->VP3 = 0;
	   		Cmd->VP2 = 0;
	   		Cmd->VP1 = 0;
	   		Cmd->VS1 = 0;
	   		Cmd->VS2 = 0;
	   		Cmd->VS3 = 0;
	   		Cmd->VS4 = 0;
	   		Cmd->VS5 = 0;
	   		Cmd->VS6 = 0;
	}//End switch

	if (Cmd->MotionType == TRANS)
	{
		if(RxByte & 0x04)
		{
			Cmd->Direction = BWD;
		}

		else
		{
			Cmd->Direction = FWD;
		}
	}
	
	else 
	{
		if(RxByte & 0x04) 
		{
			Cmd->Direction = CCW;
		}
		
		else 
		{
			Cmd->Direction = CW;
		}
	}

	Cmd->Status = NEW;

	return;

}// End ProcessRxByte

//////////////////////////////////////////////////////////////////////////////////////
/*
 Routine to implement a PID control loop
*/

int updatePID(PIDstruct * pid, float error, float position)
{
  float pTerm = 0.0,
  		dTerm = 0.0,
  		iTerm = 0.0;
  float value = 0.0;

  pTerm = pid->pGain * error;   // calculate the proportional term

// calculate the integral state with appropriate limiting

  pid->iState = pid->iState + error;
  if (pid->iState > pid->iMax)
  {
	  pid->iState = pid->iMax;
  }

  else if (pid->iState < pid->iMin)
  {
	  pid->iState = pid->iMin;
  }

  iTerm = pid->iGain * pid->iState;  // calculate the integral term

  dTerm = pid->dGain * (pid->dState - position);
  pid->dState = position;

  value = (pTerm + iTerm + dTerm + 0.5);
  if (value > PWM_MAX)
  {
	  value = PWM_MAX;
  }

  if (value < PWM_MIN)
  {
	  value = PWM_MIN;
  }

  return ((int) value);
}

/////////////////////////////////////////////////////////////////////////////

void ControlWheel(CmdStruct * Cmd, WheelStruct * Wheel, PIDstruct * PID)
{
	WORD	wReadCounter = 0;
	int		error = 0;
	int		deltaCount = 0;
	BYTE	LS_MASK = 0;
	BOOL	Start;

	if (Cmd->Status == NEW)
	{
		Counter16_1_Stop();
		Counter16_1_WritePeriod(PRESET_CNT);
		Counter16_1_Start();
		Counter16_2_Stop();
		Counter16_2_WritePeriod(PRESET_CNT);
		Counter16_2_Start();

		if (Cmd->MotionType == TRANS)
		{
			if(Wheel->id == SOUTH)
			{
				Wheel->direction = Cmd->Direction;
				Wheel->Velocity = Cmd->Velocity;
			}

			else
			{
				Wheel->direction = Cmd->Direction;
				Wheel->Velocity = Cmd->Velocity;
			}
		}
		
		if (Cmd->MotionType == ROT)
		{
			if(Cmd->Direction == CW)
			{
				if(Wheel->id == SOUTH)
				{
					Wheel->direction = FWD;
					Wheel->Velocity = Cmd->Velocity;					
				}
				
				else
				{
					Wheel->direction = BWD;
					Wheel->Velocity = Cmd->Velocity;	
				}
			}// end CW
			
			else	// CCW
			{	
				if(Wheel->id == SOUTH)
				{
					Wheel->direction = BWD;
					Wheel->Velocity = Cmd->Velocity;	
				}
				
				else
				{
					Wheel->direction = FWD;
					Wheel->Velocity = Cmd->Velocity;	
				}
			}	// end CCW
		}	// end ROT

		if (Wheel->id == NORTH)
		{
			Cmd->Status = OLD;
		}
    }

    else
    {
    
    	if (Wheel->id == SOUTH)
		{
			Cmd->wReadCounter = Counter16_1_wReadCounter();
		}
		
		if (Wheel->id == NORTH)
		{
			Cmd->wReadCounter = Counter16_2_wReadCounter();
		}
						
		deltaCount = (Wheel->position - Cmd->wReadCounter);
		Cmd->error = Wheel->Velocity - deltaCount;
		Cmd->PWMvalue = updatePID(PID, Cmd->error, Wheel->position);
		Wheel->position = Cmd->wReadCounter;
		
 		if (Cmd->LineFollow)
		{
			LS_MASK = (PRT2DR | IS_LN_MSK);

			switch (LS_MASK)
			{
				case NS_LS1_ON:
					if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VS6;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP6;
		   			}
			   			
//		   			Cmd->END_SENSOR = TRUE;
		    	break;
				    
		    	case NS_LS12_ON:
		    		if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VS5;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP5;
		   			}
		   			
//		   			Cmd->END_SENSOR = FALSE;
		    	break;

		   		case NS_LS2_ON:
		   			if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VS4;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP4;
		   			}
			   			
//		   			Cmd->END_SENSOR = FALSE;
		    	break;
				   
		    	case NS_LS23_ON:
		    		if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VS3;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP3;
		   			}
		   			
//		   			Cmd->END_SENSOR = FALSE;
		   		break;

		   		case NS_LS3_ON:
		   			if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VS2;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP2;
		   			}
		   			
//		   			Cmd->END_SENSOR = FALSE;
		    	break;
				    
		    	case NS_LS34_ON:
		    		if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VS1;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VP1;
		   			}
			   			
//		   			Cmd->END_SENSOR = FALSE;
		    	break;

				case NS_LS4_ON:
//		   			Wheel->Velocity = Cmd->Velocity;
//		   			Cmd->END_SENSOR = FALSE;
		    	break;
			    
		    	case NS_LS45_ON:
		    		if (Wheel->id == SOUTH)
					{
		   				Wheel->Velocity = Cmd->VP1;
		   			}
			   			
		   			else
		   			{
		   				Wheel->Velocity = Cmd->VS1;
		   			}
			   			
//		   			Cmd->END_SENSOR = FALSE;
		    	break;

	   			case NS_LS5_ON:
	   				if (Wheel->id == SOUTH)
					{
	   					Wheel->Velocity = Cmd->VP2;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS2;
		   			}
		   				
//	   				Cmd->END_SENSOR = FALSE;
	    		break;
			    	
	    		case NS_LS56_ON:
	    			if (Wheel->id == SOUTH)
					{
	   					Wheel->Velocity = Cmd->VP3;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS3;
		   			}
		   				
//	   				Cmd->END_SENSOR = FALSE;
	    		break;

	   			case NS_LS6_ON:
	   				if (Wheel->id == SOUTH)
					{
	   					Wheel->Velocity = Cmd->VP4;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS4;
		   			}
	   				
//	   				Cmd->END_SENSOR = FALSE;
	    		break;
			    	
	    		case NS_LS67_ON:
	    			if (Wheel->id == SOUTH)
					{
	   					Wheel->Velocity = Cmd->VP5;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS5;
		   			}
		   				
//		   			Cmd->END_SENSOR = FALSE;
	    		break;

	   			case NS_LS7_ON:
	   				if (Wheel->id == SOUTH)
					{
	   					Wheel->Velocity = Cmd->VP6;
	   				}
		   				
	   				else
		   			{
		   				Wheel->Velocity = Cmd->VS6;
		   			}
		   				
//		   			Cmd->END_SENSOR = TRUE;
	    		break;
		    		
	    		case ALL_OFF:
	    			if (Cmd->END_SENSOR)
	    			{
	    				Wheel->Velocity = 0;
	    				Cmd->Velocity = 0;
	    			}
						
					else
					{
	    				Wheel->Velocity = Wheel->Velocity;
//	    				Cmd->END_SENSOR = FALSE;
	    			}
	    		break;

				default:
		   				Wheel->Velocity = Wheel->Velocity;
//	    				Cmd->END_SENSOR = FALSE;
			}//End Switch
		}
		
		Motor(Wheel->id, Cmd->PWMvalue, Wheel->direction);
	}

	return;
}// End ControlWheel